--[[
ModSettingsManager

Helper class for ModSettings.

	@author: 	BayernGamers
	@date: 		26.12.2024
	@version:	2.0

	History:	v1.0 @05.12.2023 - initial implementation in FS 22
				--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                v2.0 @26.12.2024 - convert to FS 25

    License:    Terms:
                    Usage:
                        Feel free to use this work as-is as long as you adhere to the following terms:
                    Attribution:
                        You must give appropriate credit to the original author when using this work.
                    No Derivatives:
                        You may not alter, transform, or build upon this work in any way.
                    Usage:
                        The work may be used for personal and commercial purposes, provided it is not modified or adapted.
                    Additional Clause:
                        This script may not be converted, adapted, or incorporated into any other game versions or platforms except by GIANTS Software.
]]

source(Utils.getFilename("scripts/utils/LoggingUtil.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/utils/ModularHUDUtil.lua", g_currentModDirectory))

local log = LoggingUtil.new(true, LoggingUtil.DEBUG_LEVELS.HIGH, "ModSettingsManager.lua")

--- ModSettingsManager
-- Helper class for ModSettings.
-- @classmod      ModSettingsManager
-- @field      instance ModSettingsManager The singleton instance of the ModSettingsManager.
-- @field      debugLevel number The debug level for logging.
-- @field      debugLevelHigh number The high debug level for logging.
ModSettingsManager = {}
ModSettingsManager.instance = nil
ModSettingsManager.debugLevel = 1
ModSettingsManager.debugLevelHigh = 2

local isDedi = g_dedicatedServer ~= nil

if isDedi then
    log:printDevInfo("Running on dedicated server, skipping script", ModSettingsManager.debugLevelHigh)
    return
end

local ModSettingsManager_mt = Class(ModSettingsManager)

--- Get instance of ModSettingsManager
-- @return     ModSettingsManager The instance of the ModSettingsManager.
function ModSettingsManager.getInstance()
    if ModSettingsManager.instance == nil then
        ModSettingsManager.instance = new(g_currentModDirectory, g_currentModSettingsDirectory)
    end

    return ModSettingsManager.instance
end


--- Create new instance of ModSettingsManager
-- @param modDirectory directory of the mod
-- @param modSettingsDirectory directory of the mod settings
-- @return the new instance
function new(modDirectory, modSettingsDirectory)
    local instance = setmetatable({}, ModSettingsManager_mt)

    instance.modSettingsDirectory = modSettingsDirectory
    instance.modDirectory = modDirectory
    instance.modSettingsXMLFile = instance.modSettingsDirectory .. "modSettings.xml"
    instance.showMiniMapKey = "ModularHUD.showMiniMap#enabled"
    instance.showTachometerKey = "ModularHUD.showTachometer#enabled"
    instance.showTooltipsKey = "ModularHUD.showTooltips#enabled"
    instance.showGameInfoKey = "ModularHUD.showGameInfo#enabled"
    instance.showInfoDisplayKey = "ModularHUD.showInfoDisplay#enabled"
    instance.showAimOverlayKey = "ModularHUD.showAimOverlay#enabled"
    instance.showInfoDisplayNoHUDKey = "ModularHUD.showInfoDisplayNoHUD#enabled"
    instance.showAimOverlayNoHUDKey = "ModularHUD.showAimOverlayNoHUD#enabled"

    instance.showMiniMap = true
    instance.showTachometer = true
    instance.showTooltips = true
    instance.showGameInfo = true
    instance.showInfoDisplay = true
    instance.showAimOverlay = true
    instance.showInfoDisplayNoHUD = true
    instance.showAuimOverlayNoHUD = true

    instance.modSettingsXMLContent = [[
        <?xml version="1.0" encoding="utf-8" standalone="no"?>
        <ModularHUD>
            <showMiniMap enabled="]] .. tostring(instance.showMiniMap) .. [["/>
            <showTachometer enabled="]] .. tostring(instance.showTachometer) .. [["/>
            <showTooltips enabled="]] .. tostring(instance.showTooltips) .. [["/>
            <showGameInfo enabled="]] .. tostring(instance.showGameInfo) .. [["/>
            <showInfoDisplay enabled="]] .. tostring(instance.showInfoDisplay) .. [["/>
            <showAimOverlay enabled="]] .. tostring(instance.showAimOverlay) .. [["/>
            <showInfoDisplayNoHUD enabled="]] .. tostring(instance.showInfoDisplayNoHUD) .. [["/>
            <showAimOverlayNoHUD enabled="]] .. tostring(instance.showAimOverlayNoHUD) .. [["/>
        </ModularHUD>
        ]]

    createFolder(instance.modSettingsDirectory)

    if not fileExists(instance.modSettingsXMLFile) then
        ModularHUDUtil.writeFile(instance.modSettingsXMLFile, instance.modSettingsXMLContent)
        instance.modSettingsXML = loadXMLFile("modularHUD", instance.modSettingsXMLFile)
    else
        instance.modSettingsXML = loadXMLFile("modularHUD", instance.modSettingsXMLFile)
        instance.showMiniMap = getXMLBool(instance.modSettingsXML, instance.showMiniMapKey)
        instance.showTachometer = getXMLBool(instance.modSettingsXML, instance.showTachometerKey)
        instance.showTooltips = getXMLBool(instance.modSettingsXML, instance.showTooltipsKey)
        instance.showGameInfo = getXMLBool(instance.modSettingsXML, instance.showGameInfoKey)
        instance.showInfoDisplay = getXMLBool(instance.modSettingsXML, instance.showInfoDisplayKey)
        instance.showAimOverlay = getXMLBool(instance.modSettingsXML, instance.showAimOverlayKey)
        instance.showInfoDisplayNoHUD = getXMLBool(instance.modSettingsXML, instance.showInfoDisplayNoHUDKey)
        instance.showAimOverlayNoHUD = getXMLBool(instance.modSettingsXML, instance.showAimOverlayNoHUDKey)
    end

    return instance
end

--- Save the mod settings
-- @return     void
function ModSettingsManager:save()
    log:printDevInfo("Saving mod settings", ModSettingsManager.debugLevelHigh)
    if not fileExists(self.modSettingsXMLFile) then
        log:printDevInfo("Creating mod settings file", ModSettingsManager.debugLevelHigh)
        ModularHUDUtil.writeFile(self.modSettingsXMLFile, self.modSettingsXMLContent)
    end

    if self.showMiniMap == nil then
        self.showMiniMap = true
    end

    if self.showTachometer == nil then
        self.showTachometer = true
    end

    if self.showTooltips == nil then
        self.showTooltips = true
    end

    if self.showGameInfo == nil then
        self.showGameInfo = true
    end

    if self.showInfoDisplay == nil then
        self.showInfoDisplay = true
    end

    if self.showAimOverlay == nil then
        self.showAimOverlay = true
    end

    if self.showInfoDisplayNoHUD == nil then
        self.showInfoDisplayNoHUD = true
    end

    if self.showAimOverlayNoHUD == nil then
        self.showAimOverlayNoHUD = true
    end

    log:printDevInfo("Saving miniMap setting: " .. tostring(self.showMiniMap), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showMiniMapKey, self.showMiniMap)

    log:printDevInfo("Saving tachometer setting: " .. tostring(self.showTachometer), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showTachometerKey, self.showTachometer)

    log:printDevInfo("Saving tooltips setting: " .. tostring(self.showTooltips), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showTooltipsKey, self.showTooltips)

    log:printDevInfo("Saving game info setting: " .. tostring(self.showGameInfo), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showGameInfoKey, self.showGameInfo)

    log:printDevInfo("Saving info display setting: " .. tostring(self.showInfoDisplay), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showInfoDisplayKey, self.showInfoDisplay)

    log:printDevInfo("Saving aim overlay setting: " .. tostring(self.showAimOverlay), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showAimOverlayKey, self.showAimOverlay)

    log:printDevInfo("Saving info display no HUD setting: " .. tostring(self.showInfoDisplayNoHUD), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showInfoDisplayNoHUDKey, self.showInfoDisplayNoHUD)

    log:printDevInfo("Saving aim overlay no HUD setting: " .. tostring(self.showAimOverlayNoHUD), ModSettingsManager.debugLevelHigh)
    setXMLBool(self.modSettingsXML, self.showAimOverlayNoHUDKey, self.showAimOverlayNoHUD)

    log:printDevInfo("Saving mod settings file", ModSettingsManager.debugLevelHigh)
    saveXMLFile(self.modSettingsXML)
end

--- Load the mod settings
-- @return     void
function ModSettingsManager:load()
    if fileExists(self.modSettingsXMLFile) then
        self.modSettingsXML = loadXMLFile("modularHUD", self.modSettingsXMLFile)
        self.showMiniMap = getXMLBool(self.modSettingsXML, self.showMiniMapKey)
        self.showTachometer = getXMLBool(self.modSettingsXML, self.showTachometerKey)
        self.showTooltips = getXMLBool(self.modSettingsXML, self.showTooltipsKey)
        self.showGameInfo = getXMLBool(self.modSettingsXML, self.showGameInfoKey)
        self.showInfoDisplay = getXMLBool(self.modSettingsXML, self.showInfoDisplayKey)
        self.showAimOverlay = getXMLBool(self.modSettingsXML, self.showAimOverlayKey)
        self.showInfoDisplayNoHUD = getXMLBool(self.modSettingsXML, self.showInfoDisplayNoHUDKey)
        self.showAimOverlayNoHUD = getXMLBool(self.modSettingsXML, self.showAimOverlayNoHUDKey)
    end
end

--- Get the showMiniMap setting
-- @return     boolean The showMiniMap setting.
function ModSettingsManager:getShowMiniMap()
    return self.showMiniMap
end

--- Set the showMiniMap setting
-- @param      showMiniMap boolean The new showMiniMap setting.
-- @return     void
function ModSettingsManager:setShowMiniMap(showMiniMap)
    self.showMiniMap = showMiniMap
    self:save()
end

--- Get the showTachometer setting
-- @return     boolean The showTachometer setting.
function ModSettingsManager:getShowTachometer()
    return self.showTachometer
end

--- Set the showTachometer setting
-- @param      showTachometer boolean The new showTachometer setting.
-- @return     void
function ModSettingsManager:setShowTachometer(showTachometer)
    self.showTachometer = showTachometer
    self:save()
end

--- Get the showTooltips setting
-- @return     boolean The showTooltips setting.
function ModSettingsManager:getShowTooltips()
    return self.showTooltips
end

--- Set the showTooltips setting
-- @param      showTooltips boolean The new showTooltips setting.
-- @return     void
function ModSettingsManager:setShowTooltips(showTooltips)
    self.showTooltips = showTooltips
    self:save()
end

--- Get the showGameInfo setting
-- @return     boolean The showGameInfo setting.
function ModSettingsManager:getShowGameInfo()
    return self.showGameInfo
end

--- Set the showGameInfo setting
-- @param      showGameInfo boolean The new showGameInfo setting.
-- @return     void
function ModSettingsManager:setShowGameInfo(showGameInfo)
    self.showGameInfo = showGameInfo
    self:save()
end

--- Get the showInfoDisplay setting
-- @return     boolean The showInfoDisplay setting.
function ModSettingsManager:getShowInfoDisplay()
    return self.showInfoDisplay
end

--- Set the showInfoDisplay setting
-- @param      showInfoDisplay boolean The new showInfoDisplay setting.
-- @return     void
function ModSettingsManager:setShowInfoDisplay(showInfoDisplay)
    self.showInfoDisplay = showInfoDisplay
    self:save()
end

--- Get the showAimOverlay setting
-- @return     boolean The showAimOverlay setting.
function ModSettingsManager:getShowAimOverlay()
    return self.showAimOverlay
end

--- Set the showAimOverlay setting
-- @param      showAimOverlay boolean The new showAimOverlay setting.
-- @return     void
function ModSettingsManager:setShowAimOverlay(showAimOverlay)
    self.showAimOverlay = showAimOverlay
    self:save()
end

--- Get the showInfoDisplayNoHUD setting
-- @return     boolean The showInfoDisplayNoHUD setting.
function ModSettingsManager:getShowInfoDisplayNoHUD()
    return self.showInfoDisplayNoHUD
end

--- Set the showInfoDisplayNoHUD setting
-- @param      showInfoDisplayNoHUD boolean The new showInfoDisplayNoHUD setting.
-- @return     void
function ModSettingsManager:setShowInfoDisplayNoHUD(showInfoDisplayNoHUD)
    self.showInfoDisplayNoHUD = showInfoDisplayNoHUD
    self:save()
end

--- Get the showAimOverlayNoHUD setting
-- @return     boolean The showAimOverlayNoHUD setting.
function ModSettingsManager:getShowAimOverlayNoHUD()
    return self.showAimOverlayNoHUD
end

--- Set the showAimOverlayNoHUD setting
-- @param      showAimOverlayNoHUD boolean The new showAimOverlayNoHUD setting.
-- @return     void
function ModSettingsManager:setShowAimOverlayNoHUD(showAimOverlayNoHUD)
    self.showAimOverlayNoHUD = showAimOverlayNoHUD
    self:save()
end

