--[[
    ModularHUD

    Modular HUD for Farming Simulator 25.

        @author:    BayernGamers
        @date:      26.12.2024
        @version:   2.0

        History:    v1.0 @17.12.2023 - initial implementation in FS22
                    -------------------------------------------------------------------------------------------
                    v2.0 @26.12.2024 - convert to FS25, cleanup and refactoring

        License:    Terms:
                        Usage:
                            Feel free to use this work as-is as long as you adhere to the following terms:
                        Attribution:
                            You must give appropriate credit to the original author when using this work.
                        No Derivatives:
                            You may not alter, transform, or build upon this work in any way.
                        Usage:
                            The work may be used for personal and commercial purposes, provided it is not modified or adapted.
                        Additional Clause:
                            This script may not be converted, adapted, or incorporated into any other game versions or platforms except by GIANTS Software.
]]

source(Utils.getFilename("scripts/utils/LoggingUtil.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/utils/ModularHUDUtil.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/ModSettingsManager.lua", g_currentModDirectory))

local log = LoggingUtil.new(true, LoggingUtil.DEBUG_LEVELS.HIGH, "ModularHUD.lua")

--- Modular HUD
-- Contains functions to toggle the HUD visibility.
-- @classmod      ModularHUD
-- @field      debugLevel number The debug level for logging.
-- @field      debugLevelHigh number The high debug level for logging.
-- @field      MOD_DIRECTORY string The directory of the mod.
-- @field      MOD_NAME string The name of the mod.
-- @field      MOD_SETTINGS_DIRECTORY string The directory for mod settings.
ModularHUD = {}

ModularHUD.debugLevel = 1
ModularHUD.debugLevelHigh = 2
ModularHUD.MOD_DIRECTORY = g_currentModDirectory;
ModularHUD.MOD_NAME = g_currentModName;
ModularHUD.MOD_SETTINGS_DIRECTORY = g_currentModSettingsDirectory;

local isDedi = g_dedicatedServer ~= nil

if isDedi then
    log:printDevInfo("Running on dedicated server, skipping script", ModularHUD.debugLevelHigh)
    return
end

-- Initialize ModSettingsManager
local modSettingsManager = ModSettingsManager.getInstance()

--- Loads the map and sets HUD visibility.
-- @param      savegame table The savegame data.
-- @return     void
function ModularHUD:loadMap(savegame)
    g_currentMission.hud.hudVisibility = true
end

-- Registers action events for vehicles.
-- @return     void
function ModularHUD:registerActionEvents()
    log:printDevInfo("Registering action events", ModularHUD.debugLevelHigh)
    local valid, eventId, _ = g_inputBinding:registerActionEvent(InputAction.MH_SwitchHUDState, InputBinding.NO_EVENT_TARGET, ModularHUD.switchHUDState, false, true, false, true)
    log:printDevInfo("Action EventId: " .. tostring(eventId), ModularHUD.debugLevelHigh)
    if valid then
        g_inputBinding:setActionEventText(eventId, g_i18n:getText("input_MH_SwitchHUDState"))
        g_inputBinding:setActionEventTextVisibility(eventId, true)
        g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_VERYLOW)
    end
end
Enterable.onRegisterActionEvents = Utils.appendedFunction(Enterable.onRegisterActionEvents, ModularHUD.registerActionEvents)

--- Registers player action events if the player is the owner.
-- @return     void
function ModularHUD:registerPlayerActionEvents()
    if self.player.isOwner then
        g_inputBinding:beginActionEventsModification(PlayerInputComponent.INPUT_CONTEXT_NAME)

        PlayerInputComponent.MH_EVENTS = {}

        log:printDevInfo("Registering player action events", ModularHUD.debugLevelHigh)
        local valid, eventId = g_inputBinding:registerActionEvent(InputAction.MH_SwitchHUDState, ModularHUD, ModularHUD.switchHUDState, false, true, false, true, nil, true)

        log:printDevInfo("Player Action EventId: " .. tostring(eventId), ModularHUD.debugLevelHigh)
        if valid then
            table.insert(PlayerInputComponent.MH_EVENTS, eventId)
            g_inputBinding:setActionEventText(eventId, g_i18n:getText("input_MH_SwitchHUDState"))
            g_inputBinding:setActionEventTextVisibility(eventId, true)
            g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_VERYLOW)
        end
        g_inputBinding:endActionEventsModification()
    end
end
PlayerInputComponent.registerActionEvents = Utils.appendedFunction(PlayerInputComponent.registerActionEvents, ModularHUD.registerPlayerActionEvents)

--- Toggles the visibility of the HUD.
-- @return     void
function ModularHUD:switchHUDState()
    log:printDevInfo("Toggling HUD visibility", ModularHUD.debugLevelHigh)
    g_currentMission.hud.hudVisibility = not g_currentMission.hud.hudVisibility
end

--- Adds an action event to the vehicle. (appended)
-- @param      superFunc function The original function to be overwritten.
-- @param      actionEventsTable table The table of action events.
-- @param      inputAction string The input action.
-- @param      target table The target object.
-- @param      callback function The callback function.
-- @param      triggerUp boolean Whether the event is triggered on key up.
-- @param      triggerDown boolean Whether the event is triggered on key down.
-- @param      triggerAlways boolean Whether the event is always triggered.
-- @param      startActive boolean Whether the event starts active.
-- @param      callbackState table The state for the callback.
-- @param      customIconName string The custom icon name.
-- @param      ignoreCollisions boolean Whether to ignore collisions.
-- @param      reportAnyDeviceCollision boolean Whether to report any device collision.
-- @return     boolean, number The state and action event ID.
function ModularHUD:addActionEvent(superFunc, actionEventsTable, inputAction, target, callback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName, ignoreCollisions, reportAnyDeviceCollision)
	local state, actionEventId, otherEvents = g_inputBinding:registerActionEvent(inputAction, target, callback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, true, reportAnyDeviceCollision)

	if state then
		actionEventsTable[inputAction] = {
			actionEventId = actionEventId
		}
		local event = g_inputBinding.events[actionEventId]

		if event ~= nil then
			event.parentEventsTable = actionEventsTable
		end

		if customIconName and customIconName ~= "" then
			g_inputBinding:setActionEventIcon(actionEventId, customIconName)
		end
	end

	if otherEvents ~= nil and (ignoreCollisions == nil or not ignoreCollisions) then
		if self:getIsSelected() then
			local clearedVehicleEvent = false

			for _, otherEvent in ipairs(otherEvents) do
				if otherEvent.parentEventsTable ~= nil then
					g_inputBinding:removeActionEvent(otherEvent.id)

					otherEvent.parentEventsTable[otherEvent.id] = nil
					clearedVehicleEvent = true
				end
			end

			if clearedVehicleEvent then
				return self:addActionEvent(actionEventsTable, inputAction, target, callback, triggerUp, triggerDown, triggerAlways, startActive, callbackState, customIconName)
			end
		else
			g_inputBinding:removeActionEvent(actionEventId)

			for _, otherEvent in ipairs(otherEvents) do
                if otherEvent.targetObject ~= InputBinding.NO_EVENT_TARGET then
                    if otherEvent.targetObject.getIsSelected ~= nil and not otherEvent.targetObject:getIsSelected() and otherEvent.parentEventsTable ~= nil then
                        g_inputBinding:removeActionEvent(otherEvent.id)

                        otherEvent.parentEventsTable[otherEvent.id] = nil
                    end
                end
			end
		end
	end

	return state, actionEventId
end

Vehicle.addActionEvent = Utils.overwrittenFunction(Vehicle.addActionEvent, ModularHUD.addActionEvent)

--- Draws the input help on the HUD. (overwritten)
-- @return     void
function ModularHUD:drawInputHelp(superFunc, ...)
    if g_currentMission.hud.hudVisibility then
        if modSettingsManager:getShowTooltips() then
            superFunc(self, ...)
        end
    end

    if g_constructionScreen.isOpen then
        superFunc(self, ...)
    end
end

HUD.drawInputHelp = Utils.overwrittenFunction(HUD.drawInputHelp, ModularHUD.drawInputHelp)

--- Draws the controlled entity HUD. (overwritten)
-- @return     void
function ModularHUD:drawControlledEntityHUD()
    if self.hudVisibility then
        if self.isVisible then
            if not self.isControllingPlayer then
                if self.showVehicleInfo then
                    self.controlledVehicle:draw()
                end
            elseif self.player ~= nil then
                self.player:draw()
                if modSettingsManager:getShowInfoDisplay() then
                    self.infoDisplay:draw()
                end
            end

            if modSettingsManager:getShowTachometer() then
                self.fillLevelsDisplay:draw()
                self.speedMeter:draw()
            end
            self.contextActionDisplay:draw()
        end
    else
        if self.controlledVehicle ~= nil then
            if self.showVehicleInfo then
                self.controlledVehicle:draw()
            end
        elseif self.isControllingPlayer and self.player ~= nil then
            self.player:draw()

            if modSettingsManager:getShowInfoDisplayNoHUD() then
                self.infoDisplay:draw()
            end
        end
    end
end

HUD.drawControlledEntityHUD = Utils.overwrittenFunction(HUD.drawControlledEntityHUD, ModularHUD.drawControlledEntityHUD)

--- Draws the communication display on the HUD. (overwritten)
-- @return     void
function ModularHUD:drawCommunicationDisplay()
    if self.hudVisibility then
        if self.isVisible and not self:getIsFading() then
            self.chatDisplay:draw()
            self.speakerDisplay:draw()
        end
    end
end

HUD.drawCommunicationDisplay = Utils.overwrittenFunction(HUD.drawCommunicationDisplay, ModularHUD.drawCommunicationDisplay)

--- Draws the base HUD elements. (overwritten)
-- @return     void
function ModularHUD:drawBaseHUD()
    if self.hudVisibility then
        if modSettingsManager:getShowMiniMap() then
            self.ingameMap:draw()
        end
        if modSettingsManager:getShowGameInfo() then
            self.gameInfoDisplay:draw()
        end
        self:drawSideNotification()
        self.achievementMessage:draw()
    end
end

HUD.drawBaseHUD = Utils.overwrittenFunction(HUD.drawBaseHUD, ModularHUD.drawBaseHUD)

--- Draws the aim overlay on the HUD. (overwritten)
-- @param      superFunc function The original function to be overwritten.
-- @return     void
function ModularHUD:drawAimOverlay(superFunc)

    if g_currentMission.hud.hudVisibility and modSettingsManager:getShowAimOverlay() then
        superFunc(self)
    elseif not  g_currentMission.hud.hudVisibility and modSettingsManager:getShowAimOverlayNoHUD() then
        superFunc(self)
    end
end

HandTool.draw = Utils.overwrittenFunction(HandTool.draw, ModularHUD.drawAimOverlay)