--[[
    Modular HUD Settings

    Adds the Modular HUD settings to the game settings menu.

        @author:    BayernGamers
        @date:      26.12.2024
        @version:   2.0

        History:    v1.0 @17.12.2023 - initial implementation in FS22
                    -------------------------------------------------------------------------------------------
                    v2.0 @26.12.2024 - convert to FS25, cleanup and refactoring

        License:    Terms:
                        Usage:
                            Feel free to use this work as-is as long as you adhere to the following terms:
						Attribution:
							You must give appropriate credit to the original author when using this work.
						No Derivatives:
							You may not alter, transform, or build upon this work in any way.
						Usage:
							The work may be used for personal and commercial purposes, provided it is not modified or adapted.
						Additional Clause:
							This script may not be converted, adapted, or incorporated into any other game versions or platforms except by GIANTS Software.
]]

source(Utils.getFilename("scripts/utils/LoggingUtil.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/ModSettingsManager.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/utils/ModularHUDUtil.lua", g_currentModDirectory))

local log = LoggingUtil.new(true, LoggingUtil.DEBUG_LEVELS.HIGH, "ModularHUDSettings.lua")

--- Modular HUD Settings
-- Adds the Modular HUD settings to the game settings menu.
-- @classmod      ModularHUDSettings
-- @field      debugLevel number The debug level for logging.
-- @field      debugLevelHigh number The high debug level for logging.
-- @field      MOD_DIRECTORY string The directory of the mod.
-- @field      MOD_NAME string The name of the mod.
-- @field      MOD_SETTINGS_DIRECTORY string The directory for mod settings.
ModularHUDSettings = {}

ModularHUDSettings.debugLevel = 1
ModularHUDSettings.debugLevelHigh = 2
ModularHUDSettings.MOD_DIRECTORY = g_currentModDirectory
ModularHUDSettings.MOD_NAME = g_currentModName
ModularHUDSettings.MOD_SETTINGS_DIRECTORY = g_currentModSettingsDirectory

local isDedi = g_dedicatedServer ~= nil

if isDedi then
    log:printDevInfo("Running on dedicated server, skipping script", ModularHUDSettings.debugLevelHigh)
    return
end

--- Initializes the Modular HUD settings and adds them to the game settings menu.
-- @return     void
function ModularHUDSettings.init()

    local modSettingsManager = ModSettingsManager.getInstance()
    local settingsLayout = g_inGameMenu.generalSettingsLayout

    ModularHUDUtil.addTitle(g_i18n:getText("title_modularHUD"))

    local showMiniMapState = modSettingsManager:getShowMiniMap() and 2 or 1
    ModularHUDUtil.addCheckBox("showMiniMap", g_i18n:getText("MH_settingMM"), g_i18n:getText("MH_toolTipMM"), showMiniMapState, "onShowMiniMapChanged")

    local showTachometerState = modSettingsManager:getShowTachometer() and 2 or 1
    ModularHUDUtil.addCheckBox("showTachometer", g_i18n:getText("MH_settingTA"), g_i18n:getText("MH_toolTipTA"), showTachometerState, "onShowTachometerChanged")

    local showTooltipsState = modSettingsManager:getShowTooltips() and 2 or 1
    ModularHUDUtil.addCheckBox("showTooltips", g_i18n:getText("MH_settingTT"), g_i18n:getText("MH_toolTipTT"), showTooltipsState, "onShowTooltipsChanged")

    local showGameInfoState = modSettingsManager:getShowGameInfo() and 2 or 1
    ModularHUDUtil.addCheckBox("showGameInfo", g_i18n:getText("MH_settingGI"), g_i18n:getText("MH_toolTipGI"), showGameInfoState, "onShowGameInfoChanged")

    local showInfoDisplayState = modSettingsManager:getShowInfoDisplay() and 2 or 1
    ModularHUDUtil.addCheckBox("showInfoDisplay", g_i18n:getText("MH_settingID"), g_i18n:getText("MH_toolTipID"), showInfoDisplayState, "onShowInfoDisplay")

    local showAimOverlayState = modSettingsManager:getShowAimOverlay() and 2 or 1
    ModularHUDUtil.addCheckBox("showAimOverlay", g_i18n:getText("MH_settingAO"), g_i18n:getText("MH_toolTipAO"), showAimOverlayState, "onShowAimOverlay")

    local showInfoDisplayNoHUDState = modSettingsManager:getShowInfoDisplayNoHUD() and 2 or 1
    ModularHUDUtil.addCheckBox("showInfoDisplayNoHUD", g_i18n:getText("MH_settingIDNH"), g_i18n:getText("MH_toolTipIDNH"), showInfoDisplayNoHUDState, "onShowInfoDisplayNoHUD")

    local showAimOverlayNoHUDState = modSettingsManager:getShowAimOverlayNoHUD() and 2 or 1
    ModularHUDUtil.addCheckBox("showAimOverlayNoHUD", g_i18n:getText("MH_settingAONH"), g_i18n:getText("MH_toolTipAONH"), showAimOverlayNoHUDState, "onShowAimOverlayNoHUD")

    settingsLayout:invalidateLayout()
end

--- Callback function for when the "Show Mini Map" setting is changed.
-- @return     void
function ModularHUDSettings:onShowMiniMapChanged()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showMiniMap to " .. tostring(not modSettingsManager:getShowMiniMap()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowMiniMap(not modSettingsManager:getShowMiniMap())
end

--- Callback function for when the "Show Tachometer" setting is changed.
-- @return     void
function ModularHUDSettings:onShowTachometerChanged()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showTachometer to " .. tostring(not modSettingsManager:getShowTachometer()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowTachometer(not modSettingsManager:getShowTachometer())
end

--- Callback function for when the "Show Tooltips" setting is changed.
-- @return     void
function ModularHUDSettings:onShowTooltipsChanged()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showTooltips to " .. tostring(not modSettingsManager:getShowTooltips()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowTooltips(not modSettingsManager:getShowTooltips())
end

--- Callback function for when the "Show Game Info" setting is changed.
-- @return     void
function ModularHUDSettings:onShowGameInfoChanged()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showGameInfo to " .. tostring(not modSettingsManager:getShowGameInfo()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowGameInfo(not modSettingsManager:getShowGameInfo())
end

--- Callback function for when the "Show Info Display" setting is changed.
-- @return     void
function ModularHUDSettings:onShowInfoDisplay()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showInfoDisplay to " .. tostring(not modSettingsManager:getShowInfoDisplay()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowInfoDisplay(not modSettingsManager:getShowInfoDisplay())
end

--- Callback function for when the "Show Aim Overlay" setting is changed.
-- @return     void
function ModularHUDSettings:onShowAimOverlay()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showAimOverlay to " .. tostring(not modSettingsManager:getShowAimOverlay()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowAimOverlay(not modSettingsManager:getShowAimOverlay())
end

--- Callback function for when the "Show Info Display No HUD" setting is changed.
-- @return     void
function ModularHUDSettings:onShowInfoDisplayNoHUD()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showInfoDisplayNoHUD to " .. tostring(not modSettingsManager:getShowInfoDisplayNoHUD()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowInfoDisplayNoHUD(not modSettingsManager:getShowInfoDisplayNoHUD())
end

--- Callback function for when the "Show Aim Overlay No HUD" setting is changed.
-- @return     void
function ModularHUDSettings:onShowAimOverlayNoHUD()
    local modSettingsManager = ModSettingsManager.getInstance()
    log:printDevInfo("Setting showAimOverlayNoHUD to " .. tostring(not modSettingsManager:getShowAimOverlayNoHUD()), ModularHUDSettings.debugLevelHigh)
    modSettingsManager:setShowAimOverlayNoHUD(not modSettingsManager:getShowAimOverlayNoHUD())
end

