--[[
    Modular HUD Interface

    Interface for modders to interact with the Modular HUD.

        @author:    BayernGamers
        @date:      28.12.2024
        @version:   1.0

        History:    v1.0 @28.12.2024 - initial implementation in FS25
                    -------------------------------------------------------------------------------------------

        License:    Terms:
                        Usage:
                            Feel free to use this work as-is as long as you adhere to the following terms:
                        Attribution:
                            You must give appropriate credit to the original author when using this work.
                        No Derivatives:
                            You may not alter, transform, or build upon this work in any way.
                        Usage:
                            The work may be used for personal and commercial purposes, provided it is not modified or adapted.
                        Additional Clause:
                            This script may not be converted, adapted, or incorporated into any other game versions or platforms except by GIANTS Software.
]]

source(Utils.getFilename("scripts/utils/LoggingUtil.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/ModSettingsManager.lua", g_currentModDirectory))

local log = LoggingUtil.new(true, LoggingUtil.DEBUG_LEVELS.HIGH, "ModularHUDInterface.lua")

--- Modular HUD Interface
-- Interface for modders to interact with the Modular HUD.
-- Please keep in mind that all functions are usable on the client side only. Check for not being on server side using g_dedicatedServer == nil.
-- @classmod      ModularHUDInterface
-- @field      debugLevel number The debug level for logging.
-- @field      debugLevelHigh number The high debug level for logging.
-- @field      MOD_DIRECTORY string The directory of the mod.
-- @field      MOD_NAME string The name of the mod.
-- @field      MOD_SETTINGS_DIRECTORY string The directory for mod settings.
ModularHUDInterface = {}
ModularHUDInterface.debugLevel = 1
ModularHUDInterface.debugLevelHigh = 2
ModularHUDInterface.MOD_DIRECTORY = g_currentModDirectory
ModularHUDInterface.MOD_NAME = g_currentModName
ModularHUDInterface.MOD_SETTINGS_DIRECTORY = g_currentModSettingsDirectory

-- Check if script is running on dedicated server
local isDedi = g_dedicatedServer ~= nil

if isDedi then
    log:printDevInfo("Running on dedicated server, skipping script", ModularHUDInterface.debugLevelHigh)
    return
end

local modSettingsManager = ModSettingsManager.getInstance()

--- Registers a function with a conditional function.
-- @param objectType The type of the object.
-- @param funcName The name of the function.
-- @param func The function to register.
-- @param condFunc The conditional function.
-- @return void
function ModularHUDInterface.registerFunction(objectType, funcName, func, condFunc)
    if condFunc == nil then
        printCallstack()
    end

    assert(func ~= nil, "Error: Given reference for Function '" .. tostring(funcName) .. "' is 'nil'!")

    func = condFunc(func)

    SpecializationUtil.registerFunction(objectType, funcName, func)
end

--- Registers an event listener with a conditional function.
-- @param objectType The type of the object.
-- @param eventName The name of the event.
-- @param spec The specialization.
-- @param condFunc The conditional function.
-- @return void
function ModularHUDInterface.registerEventListener(objectType, eventName, spec, condFunc)
    local className = ModularHUDUtil.getClassName(spec)

    assert(objectType.eventListeners ~= nil, "Error: Invalid type for specialization '" .. tostring(className) .. "'!")

    if objectType.eventListeners[eventName] == nil then
        return
    end

    assert(spec[eventName] ~= nil, "Error: Event listener function '" .. tostring(eventName) .. "' not defined in specialization '" .. tostring(className) .. "'!")
    assert(condFunc ~= nil, "Error: Given reference for Conditional Function '" .. tostring(eventName) .. "' is 'nil'!")

    spec[eventName] = condFunc(spec[eventName])

    local found = false

    for _, registeredSpec in pairs(objectType.eventListeners[eventName]) do
        if registeredSpec == spec then
            found = true

            break
        end
    end

    assert(not found, "Error: Eventlistener for '" .. eventName .. "' already registered in specialization '" .. tostring(className) .. "'!")
    table.insert(objectType.eventListeners[eventName], spec)
end

--- Returns the ModularHUDInterface instance.
-- @return The ModularHUDInterface instance.
function ModularHUDInterface.getModularHUDInterface()
    return ModularHUDInterface
end

--- Returns whether the tachometer should be shown.
-- @return True if the tachometer should be shown, false otherwise.
function ModularHUDInterface:getShowTachometer()
    return modSettingsManager:getShowTachometer() and g_currentMission.hud.hudVisibility
end

--- Returns whether the tooltips should be shown.
-- @return True if the tooltips should be shown, false otherwise.
function ModularHUDInterface:getShowTooltips()
    return modSettingsManager:getShowTooltips() and g_currentMission.hud.hudVisibility
end

--- Returns whether the game info should be shown.
-- @return True if the game info should be shown, false otherwise.
function ModularHUDInterface:getShowGameInfo()
    return modSettingsManager:getShowGameInfo() and g_currentMission.hud.hudVisibility
end

--- Returns whether the mini map should be shown.
-- @return True if the mini map should be shown, false otherwise.
function ModularHUDInterface:getShowMiniMap()
    return modSettingsManager:getShowMiniMap() and g_currentMission.hud.hudVisibility
end

--- Returns whether the info display should be shown when HUD is not visible.
-- @return True if the info display should be shown, false otherwise.
function ModularHUDInterface:getShowInfoDisplayNoHUD()
    return modSettingsManager:getShowInfoDisplayNoHUD() and not g_currentMission.hud.hudVisibility
end

--- Returns whether the aim overlay should be shown when HUD is not visible.
-- @return True if the aim overlay should be shown, false otherwise.
function ModularHUDInterface:getShowAimOverlayNoHUD()
    return modSettingsManager:getShowAimOverlayNoHUD() and not g_currentMission.hud.hudVisibility
end

--- Returns a conditional function for game info visibility.
-- @param func The function to wrap.
-- @return The wrapped function.
function ModularHUDInterface.gameInfoVisibilityConditionalFunction(func)
    return ModularHUDUtil.gameInfoVisibilityConditionalFunction(func)
end

--- Returns a conditional function for tooltips menu visibility.
-- @param func The function to wrap.
-- @return The wrapped function.
function ModularHUDInterface.tooltipsMenuVisibilityConditionalFunction(func)
    return ModularHUDUtil.tooltipsMenuVisibilityConditionalFunction(func)
end

--- Returns a conditional function for mini map visibility.
-- @param func The function to wrap.
-- @return The wrapped function.
function ModularHUDInterface.miniMapVisibilityConditionalFunction(func)
    return ModularHUDUtil.miniMapVisibilityConditionalFunction(func)
end

--- Returns a conditional function for tachometer visibility.
-- @param func The function to wrap.
-- @return The wrapped function.
function ModularHUDInterface.tachometerVisibilityConditionalFunction(func)
    return ModularHUDUtil.tachometerVisibilityConditionalFunction(func)
end

--- Returns a general visibility conditional function.
-- @param func The function to wrap.
-- @return The wrapped function.
function ModularHUDInterface.visibilityConditionalFunction(func)
    return ModularHUDUtil.visibilityConditionalFunction(func)
end

--- Registers a tachometer visibility conditional function.
-- @param objectType The type of the object.
-- @param funcName The name of the function.
-- @param func The function to register.
-- @return void
function SpecializationUtil.registerTachometerVisibilityConditionalFunction(objectType, funcName, func)
    ModularHUDInterface.registerFunction(objectType, funcName, func, ModularHUDUtil.tachometerVisibilityConditionalFunction)
end

--- Registers a game info visibility conditional function.
-- @param objectType The type of the object.
-- @param funcName The name of the function.
-- @param func The function to register.
-- @return void
function SpecializationUtil.registerGameInfoVisibilityConditionalFunction(objectType, funcName, func)
    ModularHUDInterface.registerFunction(objectType, funcName, func, ModularHUDUtil.gameInfoVisibilityConditionalFunction)
end

--- Registers a tooltips menu visibility conditional function.
-- @param objectType The type of the object.
-- @param funcName The name of the function.
-- @param func The function to register.
-- @return void
function SpecializationUtil.registerTooltipsMenuVisibilityConditionalFunction(objectType, funcName, func)
    ModularHUDInterface.registerFunction(objectType, funcName, func, ModularHUDUtil.tooltipsMenuVisibilityConditionalFunction)
end

--- Registers a mini map visibility conditional function.
-- @param objectType The type of the object.
-- @param funcName The name of the function.
-- @param func The function to register.
-- @return void
function SpecializationUtil.registerMiniMapVisibilityConditionalFunction(objectType, funcName, func)
    ModularHUDInterface.registerFunction(objectType, funcName, func, ModularHUDUtil.miniMapVisibilityConditionalFunction)
end

--- Registers a tachometer visibility conditional event listener.
-- @param objectType The type of the object.
-- @param eventName The name of the event.
-- @param spec The specialization.
-- @return void
function SpecializationUtil.registerTachomenterVisibilityConditionalEventListener(objectType, eventName, spec)
    ModularHUDInterface.registerEventListener(objectType, eventName, spec, ModularHUDUtil.tachometerVisibilityConditionalFunction)
end

--- Registers a game info visibility conditional event listener.
-- @param objectType The type of the object.
-- @param eventName The name of the event.
-- @param spec The specialization.
-- @return void
function SpecializationUtil.registerGameInfoVisibilityConditionalEventListener(objectType, eventName, spec)
    ModularHUDInterface.registerEventListener(objectType, eventName, spec, ModularHUDUtil.gameInfoVisibilityConditionalFunction)
end

--- Registers a tooltips menu visibility conditional event listener.
-- @param objectType The type of the object.
-- @param eventName The name of the event.
-- @param spec The specialization.
-- @return void
function SpecializationUtil.registerTooltipsMenuVisibilityConditionalEventListener(objectType, eventName, spec)
    ModularHUDInterface.registerEventListener(objectType, eventName, spec, ModularHUDUtil.tooltipsMenuVisibilityConditionalFunction)
end

--- Registers a mini map visibility conditional event listener.
-- @param objectType The type of the object.
-- @param eventName The name of the event.
-- @param spec The specialization.
-- @return void
function SpecializationUtil.registerMiniMapVisibilityConditionalEventListener(objectType, eventName, spec)
    ModularHUDInterface.registerEventListener(objectType, eventName, spec, ModularHUDUtil.miniMapVisibilityConditionalFunction)
end

getfenv(0).getModularHUDInterface = ModularHUDInterface.getModularHUDInterface
getfenv(0).g_modularHUDInterface = ModularHUDInterface