--[[
    ModularHUDUtil

    A utility library for ModularHUD.

        @author:    BayernGamers
        @date:      26.12.2024
        @version:   2.0

        History:    v1.0 @17.12.2023 - initial implementation in FS22
                    -------------------------------------------------------------------------------------------
                    v2.0 @26.12.2024 - extension for and conversion to FS25

    License:    Terms:
                    Usage:
                        Feel free to use this work as-is as long as you adhere to the following terms:
                    Attribution:
                        You must give appropriate credit to the original author when using this work.
                    No Derivatives:
                        You may not alter, transform, or build upon this work in any way.
                    Usage:
                        The work may be used for personal and commercial purposes, provided it is not modified or adapted.
                    Additional Clause:
                        This script may not be converted, adapted, or incorporated into any other game versions or platforms except by GIANTS Software.
]]

source(Utils.getFilename("scripts/utils/LoggingUtil.lua", g_currentModDirectory))

local log = LoggingUtil.new(true, LoggingUtil.DEBUG_LEVELS.HIGH, "ModularHUDUtil.lua")

--- ModularHUDUtil
-- A utility library for ModularHUD.
-- @classmod      ModularHUDUtil
-- @field      debugLevel number The debug level for logging.
-- @field      debugLevelHigh number The high debug level for logging.
-- @field      MOD_DIRECTORY string The directory of the mod.
-- @field      MOD_NAME string The name of the mod.
-- @field      MOD_SETTINGS_DIRECTORY string The directory for mod settings.
ModularHUDUtil = {}

ModularHUDUtil.debugLevel = 1
ModularHUDUtil.debugLevelHigh = 2
ModularHUDUtil.MOD_DIRECTORY = g_currentModDirectory;
ModularHUDUtil.MOD_NAME = g_currentModName;
ModularHUDUtil.MOD_SETTINGS_DIRECTORY = g_currentModSettingsDirectory;

--- Split a string by a pattern
-- @param      str string The string to split.
-- @param      pattern string The pattern to split by.
-- @return     table The split string as a table.
function ModularHUDUtil.splitString(str, pattern)
	local results = {}

	if str ~= nil and str ~= "" then
		local start = 1
		local splitStart, splitEnd = string.find(str, pattern, start, true)

		while splitStart ~= nil do
			table.insert(results, string.sub(str, start, splitStart - 1))

			start = splitEnd + 1
			splitStart, splitEnd = string.find(str, pattern, start, true)
		end

		table.insert(results, string.sub(str, start))
	end

	return results
end

--- Returns the class object of the given class name
-- @param      className string The class name.
-- @return     table The class object.
function ModularHUDUtil.getClassObject(className)
    local parts = ModularHUDUtil.splitString(className, ".")
    local currentTable = _G[parts[1]]

    if type(currentTable) ~= "table" then
        return nil
    end

    for i = 2, #parts do
        currentTable = currentTable[parts[i]]

        if type(currentTable) ~= "table" then
            return nil
        end
    end

    return currentTable
end

--- Returns the class name of the given class object
-- @param      classObject table The class object.
-- @return     string The class name.
function ModularHUDUtil.getClassName(classObject)

    local g = _G

	for k, v in pairs(g) do
		if v == classObject then
			return k
		end
	end

	for customEnv, _ in pairs(g_modIsLoaded) do
		for k, v in pairs(g[customEnv]) do
			if v == classObject then
				return customEnv .. "." .. k
			end
		end
	end

	return nil
end

--- Returns a function that only executes if the HUD is visible
-- @param      func function The function to execute.
-- @return     function The conditional function.
function ModularHUDUtil.visibilityConditionalFunction(func)
    if func == nil then
        log:printError("ModularHUDUtil.visibilityConditionalFunction Argument #1 is nil!")
        return nil
    end

    return function (...)
        if g_currentMission.hud.hudVisibility then
            func(...)
        end
    end
end

--- Returns a function that only executes if the HUD is visible and the tachometer is visible
-- @param      func function The function to execute.
-- @return     function The conditional function.
function ModularHUDUtil.tachometerVisibilityConditionalFunction(func)

    if func == nil then
        log:printError("ModularHUDUtil.tachometerVisibilityConditionalFunction Argument #1 is nil!")
        return nil
    end

    source(Utils.getFilename("scripts/ModSettingsManager.lua", ModularHUDUtil.MOD_DIRECTORY))
    local modSettingsManager = ModSettingsManager.getInstance()

    return function (...)
        if g_currentMission.hud.hudVisibility and modSettingsManager:getShowTachometer() then
            func(...)
        end
    end
end

--- Returns a function that only executes if the HUD is visible and the GameInfo is visible
-- @param      func function The function to execute.
-- @return     function The conditional function.
function ModularHUDUtil.gameInfoVisibilityConditionalFunction(func)

    if func == nil then
        log:printError("ModularHUDUtil.gameInfoVisibilityConditionalFunction Argument #1 is nil!")
        return nil
    end

    source(Utils.getFilename("scripts/ModSettingsManager.lua", ModularHUDUtil.MOD_DIRECTORY))
    local modSettingsManager = ModSettingsManager.getInstance()

    return function (...)
        if g_currentMission.hud.hudVisibility and modSettingsManager:getShowGameInfo() then
            func(...)
        end
    end
end

--- Returns a function that only executes if the HUD is visible and the Tooltips Menu is visible
-- @param      func function The function to execute.
-- @return     function The conditional function.
function ModularHUDUtil.tooltipsMenuVisibilityConditionalFunction(func)

    if func == nil then
        log:printError("ModularHUDUtil.tooltipsMenuVisibilityConditionalFunction Argument #1 is nil!")
        return nil
    end

    source(Utils.getFilename("scripts/ModSettingsManager.lua", ModularHUDUtil.MOD_DIRECTORY))
    local modSettingsManager = ModSettingsManager.getInstance()

    return function (...)
        if g_currentMission.hud.hudVisibility and modSettingsManager:getShowTooltips() then
            func(...)
        end
    end
end

--- Returns a function that only executes if the HUD is visible and the Minimap is visible
-- @param      func function The function to execute.
-- @return     function The conditional function.
function ModularHUDUtil.miniMapVisibilityConditionalFunction(func)

    if func == nil then
        log:printError("ModularHUDUtil.miniMapVisibilityConditionalFunction Argument #1 is nil!")
        return nil
    end

    source(Utils.getFilename("scripts/ModSettingsManager.lua", ModularHUDUtil.MOD_DIRECTORY))
    local modSettingsManager = ModSettingsManager.getInstance()

    return function (...)
        if g_currentMission.hud.hudVisibility and modSettingsManager:getShowMiniMap() then
            func(...)
        end
    end
end

--- Write given content to file
-- @param      path string The path of the file.
-- @param      content string The content to write.
-- @return     void
function ModularHUDUtil.writeFile(path, content)
    local file = io.open(path, "w")
	if file ~= nil then
		file:write(content)
		file:close()
	end
end

--- Renews the focus ids of the given element and its children
-- @param      element table The element to renew the focus ids.
-- @return     void
function ModularHUDUtil.updateElementFocusIds(element)
    if not element then
        return
    end
    element.focusId = FocusManager:serveAutoFocusId()
    for _, child in pairs(element.elements) do
        ModularHUDUtil.updateElementFocusIds(child)
    end
end

--- Add a title to the Settings Page
-- @param      text string The text of the title.
-- @return     void
function ModularHUDUtil.addTitle(text)
    local settingsLayout = g_inGameMenu.generalSettingsLayout
    local controlsList = g_inGameMenu.controlsList

    local title = nil
    for _, element in ipairs(settingsLayout.elements) do
        if element.name == "sectionHeader" then
            title = element:clone(settingsLayout)
            title:setText(text)

            ModularHUDUtil.updateElementFocusIds(title)
            table.insert(controlsList, title)
            return
        end
    end
end

--- Adds a checkbox to the Settings Page
-- @param      id string The id of the checkbox.
-- @param      text string The text of the checkbox.
-- @param      toolTip string The tooltip of the checkbox.
-- @param      state number The state of the checkbox.
-- @param      callback string The name of the callback function as a string.
-- @return     void
function ModularHUDUtil.addCheckBox(id, text, toolTip, state, callback)
    local modSettingsManager = ModSettingsManager.getInstance()
    local settingsLayout = g_inGameMenu.generalSettingsLayout
    local checkBoxTemplate = g_inGameMenu.checkWoodHarvesterAutoCutBox
    local controlsList = g_inGameMenu.controlsList

    local checkBox = checkBoxTemplate:clone(settingsLayout)
    checkBox.id = id

    local checkBoxElement = checkBox.elements[1]
    checkBoxElement.id = id
    checkBoxElement.target = ModularHUDSettings

    checkBoxElement:setCallback("onClickCallback", callback)
    checkBoxElement:setDisabled(false)

    local toolTipElement = checkBoxElement.elements[1]
    toolTipElement:setText(toolTip)

    local settingElement = checkBox.elements[2]
    settingElement:setText(text)

    checkBoxElement:setTexts({g_i18n:getText("MH_Off"), g_i18n:getText("MH_On")})
    checkBoxElement:setState(state)

    ModularHUDUtil.updateElementFocusIds(checkBox)
    table.insert(controlsList, checkBox)
end